.libPaths("C:/Users/Public/Rlib")
setwd("C:/Users/rx01/teach/countglm2020")

library(edgeR)
#---read in data
seqdt <- read.table('demo.count.txt',header=T,check.names = F)
seqdt[1:10,1:10]
dim(seqdt)
(demat <- read.table('demo.design.txt',header=T,check.names = F) )

#-----make sure data are in the same order
summary(colnames(seqdt)[-1]==demat[,1])

#---createing design matrix based on the desigh data
design <- model.matrix(~demat$breed+demat$tissue)
design

#---create edgeR object
seqdt[1:10,1:10]
exp.y <- edgeR::DGEList(counts=seqdt[,-1], genes=seqdt[,1,drop=F])
#---check total number of genes
nrow(exp.y$genes)
#---find genes lowly expressed according to some arbitrary criteria
gene.keep <- rowSums(cpm(exp.y) > 0) >=3
summary(gene.keep)
#---filter out these genes
exp.y.filter <- exp.y[gene.keep,,keep.lib.sizes=FALSE]
#---check number genes left
nrow(exp.y.filter$genes)

#---estimate tagwise dispersions (account for the the 'dispersion')
exp.y.filter <- estimateDisp(exp.y.filter,design,robust=TRUE)
exp.y.filter$common.dispersion
plotBCV(exp.y.filter)

#---fit model
#-again, make sure your gene and design data are in the same order
fit <- glmQLFit(exp.y.filter, design,robust=TRUE)
#---check out effects 
head(fit$coefficients)
#---find the effects of interest
tissComp <- glmQLFTest(fit, coef=3)
#---view the top DE gene
topTags(tissComp)
#---short summary of down and up-regulated genes
summary(decideTests(tissComp))
#---do a nice plot
plotMD(tissComp)
#=====done with GLM with edgeR!======

#=====ordinalry lm======
#--subset expression data with 'gene.keep'
seqdt[1:10,1:10]
seqdt.sub <- seqdt[gene.keep,]
dim(seqdt.sub)
#---organise data for testing
t.seqdt.sub <- t(seqdt.sub[,-1])
colnames(t.seqdt.sub) <- seqdt.sub[,1]
t.seqdt.sub[1:10,1:10]

#---a loop of olm for every gene----
#-preparation
lmlooplist <- list()
reportNum <- seq(0,ncol(t.seqdt.sub),1000)
#-go loop
for (geneid in colnames(t.seqdt.sub)[-1]) {
if(which(colnames(t.seqdt.sub)==geneid) %in% reportNum){cat(paste0('Just did gene ',which(colnames(t.seqdt.sub)==geneid),' for ya, Mate'),sep='\n')}
#--for each gene we fit this model
mod.fit <- lm(log10(t.seqdt.sub[,colnames(t.seqdt.sub)==geneid]+1)~demat$breed+demat$tissue)
#--we then extract model coefficient from each lm result
tmp <- data.frame(GeneID=geneid,summary(mod.fit)$coefficients)
#--result clean-up
colnames(tmp)[c(3,5)] <- c('se','p')
tmp$eff<- gsub('demat\\$|\\(|)','',rownames(tmp))
rownames(tmp)<- NULL
  lmlooplist[[geneid]] <- tmp
}
#---collect results from olm loop
eff_dt <- do.call(rbind,lmlooplist)
head(eff_dt)
dim(eff_dt)

#----combine two resutls
#--extract effects for tissue from OLM
tiss_eff <- subset(eff_dt,eff=='tissuemilk')

#--extract effects for tissue from edgeR
edger.res <- data.frame(topTags(tissComp,9999999))
colnames(edger.res)[2:ncol(edger.res)] <- paste0('edger.',colnames(edger.res)[2:ncol(edger.res)])
colnames(tiss_eff)[2:ncol(tiss_eff)] <- paste0('olm.',colnames(tiss_eff)[2:ncol(tiss_eff)])

#---combine results from edgeR and OLM
cbdt <- merge(edger.res,tiss_eff[,-ncol(tiss_eff)],by='GeneID',sort=F)
head(cbdt)

#---check correlation of expression change of genes between edgeR and OLM
cor(cbdt$edger.logFC,cbdt$olm.t.value)

#---have a look at the actual data of gene expression change
plot(cbdt$edger.logFC, cbdt$olm.t.value, main="Ordinary lm effects VS edgeR effects",
     title(sub=paste0("cor = ", formatC(cor(cbdt$edger.logFC,cbdt$olm.t.value), 3, format="f"))),
     xlab="edger.logFC ", ylab="olm.t.value ", pch=19)

#---compare the distribution of p-value in two methods
boxplot(-log10(cbdt$edger.PValue) , -log10(cbdt$olm.p), data=cbdt, notch=TRUE,
        col=(c("gold","skyblue")),boxwex=0.6,las=1,
        main="edgeR p-value VS ordinary lm p-value", names=c("edgeR", "OLM"),ylab="-log10(p)")

#======If you can normalise y properly, in this case log10() on gene expression,============
#======then the general linear model is not too bad=========================================